/* getstats.cc
 *
 * ----START-LICENCE----
 * Copyright 1999,2000,2001 BrightStation PLC
 * Copyright 2002 Ananova Ltd
 * Copyright 2002 Olly Betts
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * -----END-LICENCE-----
 */

#include "om/om.h"

#include <algorithm>
#include <iostream>
#include <map>

using namespace std;

static map<om_termpos, unsigned long long> first_pos, diff_pos;
static unsigned long long n_first_pos, n_diff_pos;

static void
process_positions(OmDatabase &db, om_termname t, om_docid did)
{
    OmPositionListIterator i = db.positionlist_begin(did, t);
    OmPositionListIterator end = db.positionlist_end(did, t);
    if (i == end) return;
    om_termpos prev = *i;
    ++first_pos[prev];
    ++n_first_pos;
    while (++i != end) {
	om_termpos pos = *i;
	++diff_pos[pos - prev];
	++n_diff_pos;
	prev = pos;
    }
}

int
main(int argc, char **argv)
{
    if (argc <= 1) {
	cout << "Syntax: " << argv[0] << " DATABASE... > OUTPUT" << endl;
	exit(0);
    }

    bool error = false;
    while (*(++argv)) {
	cerr << "Database: " << *argv << endl;
	cout << "Database: " << *argv << endl;
	map<om_docid, om_doccount> first_posting, diff_posting, runs, wdf;
	map<unsigned int, om_termcount> reuse_prefix, first_lengths, term_lengths, reuse_lengths;
	first_pos.clear();
	diff_pos.clear();
	n_first_pos = n_diff_pos = 0;
	om_termcount n_first = 0;
	om_termcount n_reuse_prefix = 0;
	om_termcount n_first_lengths = 0;
	om_termcount n_term_lengths = 0;
	om_termcount n_reuse_lengths = 0;
	unsigned long long n_diff = 0;
	om_termcount n_runs = 0;
	om_termcount n_wdf = 0;
	try {
	    OmDatabase db(OmAuto__open(*argv));
	    OmTermIterator term = db.allterms_begin();
	    OmTermIterator tend = db.allterms_end();
	    om_doccount run = 1;
	    while (term != tend) {
		om_termname t = *term;
		OmPostListIterator posting = db.postlist_begin(t);
		OmPostListIterator pend = db.postlist_end(t);
		if (posting != pend) {
		    om_docid prev = *posting;
		    ++wdf[posting.get_wdf()];
		    ++n_wdf;
		    ++first_posting[prev];
		    ++n_first;
		    process_positions(db, t, prev);
		    while (++posting != pend) {
			om_docid did = *posting;
			++wdf[posting.get_wdf()];
			++n_wdf;
			om_doccount diff = did - prev;
			++diff_posting[diff];
			++n_diff;
			if (diff == 1) {
			    ++run;
			} else {
			    ++runs[run];
			    ++n_runs;
			    run = 1;
			} 
			prev = did;
			process_positions(db, t, prev);
		    }
		    ++runs[run];
		}
		++term;
	    }

	    om_doccount count = db.get_doccount();
	    om_docid docid = 1;
	    while (count) {
		try {
		    OmTermIterator i = db.termlist_begin(docid);
		    OmTermIterator end = db.termlist_end(docid);
		    if (i != end) {
			om_termname prev = *i;
			++term_lengths[prev.size()];
			++n_term_lengths;
			++first_lengths[prev.size()];
			++n_first_lengths;
			while (i != end) {
			    om_termname name = *i;
			    unsigned int common;
			    common = mismatch(prev.begin(),
					      prev.begin() + min(prev.size(), name.size()),
					      name.begin()).first - prev.begin();
			    ++reuse_prefix[common];
			    ++n_reuse_prefix;
			    ++reuse_lengths[name.size() - common];
			    ++n_reuse_lengths;
			    ++term_lengths[name.size()];
			    ++n_term_lengths;
			    ++i;
			    prev = name;
			}
		    }
		    --count;
		} catch (const OmDocNotFoundError &/*error*/) {
		    // that document must have been deleted
		}
		if (docid == (om_docid)-1) break;
		++docid;
	    }
	} catch (const OmError &e) {
	    cerr << "\nError reading database: " << e.get_msg() << endl;
	    cout << "Error reading database: " << e.get_msg() << endl;
	    error = true;
	}
	map<om_docid, om_doccount>::const_iterator i;
	cout << "First ids of posting list have distribution (total = " << n_first << "):\n";
	for (i = first_posting.begin(); i != first_posting.end(); ++i) {
	    cout << i->first << '\t' << i->second << '\n';
	}
	cout << "Diffs in posting list have distribution (total = " << n_diff << "):\n";
	for (i = diff_posting.begin(); i != diff_posting.end(); ++i) {
	    cout << i->first << '\t' << i->second << '\n';
	}
	cout << "Runs in posting list have distribution (total = " << n_runs << "):\n";
	for (i = runs.begin(); i != runs.end(); ++i) {
	    cout << i->first << '\t' << i->second << '\n';
	}
	cout << "WDFs have distribution (total = " << n_wdf << "):\n";
	for (i = wdf.begin(); i != wdf.end(); ++i) {
	    cout << i->first << '\t' << i->second << '\n';
	}
	map<om_docid, unsigned long long>::const_iterator j;
	cout << "First entries of position list have distribution (total = " << n_first_pos << "):\n";
	for (j = first_pos.begin(); j != first_pos.end(); ++j) {
	    cout << j->first << '\t' << j->second << '\n';
	}
	cout << "Diffs in position list have distribution (total = " << n_diff_pos << "):\n";
	for (j = diff_pos.begin(); j != diff_pos.end(); ++j) {
	    cout << j->first << '\t' << j->second << '\n';
	}
	map<unsigned int, om_termcount>::const_iterator k;
	cout << "Term lengths have distribution (total = " << n_term_lengths << "):\n";
	for (k = term_lengths.begin(); k != term_lengths.end(); ++k) {
	    cout << k->first << '\t' << k->second << '\n';
	}
	cout << "First lengths have distribution (total = " << n_first_lengths << "):\n";
	for (k = first_lengths.begin(); k != first_lengths.end(); ++k) {
	    cout << k->first << '\t' << k->second << '\n';
	}
	cout << "Reusable term prefixes have distribution (total = " << n_reuse_prefix << "):\n";
	for (k = reuse_prefix.begin(); k != reuse_prefix.end(); ++k) {
	    cout << k->first << '\t' << k->second << '\n';
	}
	cout << "Append lengths have distribution (total = " << n_reuse_lengths << "):\n";
	for (k = reuse_lengths.begin(); k != reuse_lengths.end(); ++k) {
	    cout << k->first << '\t' << k->second << '\n';
	}
	cout << endl;
    }

    return (error ? 1 : 0);
}
